@@ -28,12 +28,12 @@
 # POSSIBILITY OF SUCH DAMAGE.
 # ===================================================================
 
-import json
 import unittest
 from binascii import unhexlify
 
 from Cryptodome.SelfTest.st_common import list_test_cases
-from Cryptodome.Util.py3compat import tobytes, _memoryview
+from Cryptodome.SelfTest.loader import load_test_vectors_wycheproof
+from Cryptodome.Util.py3compat import tobytes
 from Cryptodome.Cipher import ChaCha20_Poly1305
 from Cryptodome.Hash import SHAKE128
 
@@ -323,10 +323,6 @@ class ChaCha20Poly1305Tests(unittest.TestCase):
 
         self.assertEqual(pt_test, self.data_128)
 
-    import sys
-    if sys.version[:3] == "2.6":
-        del test_memoryview
-
 
 class XChaCha20Poly1305Tests(unittest.TestCase):
 
@@ -629,29 +625,18 @@ class TestVectorsWycheproof(unittest.TestCase):
         self._id = "None"
 
     def load_tests(self, filename):
-        comps = "Cryptodome.SelfTest.Cipher.test_vectors.wycheproof".split(".")
-        with open(pycryptodome_filename(comps, filename), "rt") as file_in:
-            tv_tree = json.load(file_in)
 
-        class TestVector(object):
-            pass
-        result = []
+        def filter_tag(group):
+            return group['tagSize'] // 8
 
-        for group in tv_tree['testGroups']:
-            tag_size = group['tagSize'] // 8
-            for test in group['tests']:
-                tv = TestVector()
-                tv.tag_size = tag_size
+        def filter_algo(root):
+            return root['algorithm']
 
-                tv.id = test['tcId']
-                tv.comment = test['comment']
-                for attr in 'key', 'iv', 'aad', 'msg', 'ct', 'tag':
-                    setattr(tv, attr, unhexlify(test[attr]))
-                tv.valid = test['result'] != "invalid"
-                tv.warning = test['result'] == "acceptable"
-                tv.algo = tv_tree['algorithm']
-
-                result.append(tv)
+        result = load_test_vectors_wycheproof(("Cipher", "wycheproof"),
+                                               filename,
+                                               "Wycheproof ChaCha20-Poly1305",
+                                               root_tag={'algo': filter_algo},
+                                               group_tag={'tag_size': filter_tag})
         return result
 
     def setUp(self):
@@ -669,7 +654,7 @@ class TestVectorsWycheproof(unittest.TestCase):
 
     def test_encrypt(self, tv):
         self._id = "Wycheproof Encrypt %s Test #%s" % (tv.algo, tv.id)
-        
+
         try:
             cipher = ChaCha20_Poly1305.new(key=tv.key, nonce=tv.iv)
         except ValueError as e:
@@ -685,7 +670,7 @@ class TestVectorsWycheproof(unittest.TestCase):
 
     def test_decrypt(self, tv):
         self._id = "Wycheproof Decrypt %s Test #%s" % (tv.algo, tv.id)
-        
+
         try:
             cipher = ChaCha20_Poly1305.new(key=tv.key, nonce=tv.iv)
         except ValueError as e:
@@ -736,34 +721,32 @@ class TestOutput(unittest.TestCase):
         res = cipher.encrypt(pt, output=output)
         self.assertEqual(ct, output)
         self.assertEqual(res, None)
-        
+
         cipher = ChaCha20_Poly1305.new(key=key, nonce=nonce)
         res = cipher.decrypt(ct, output=output)
         self.assertEqual(pt, output)
         self.assertEqual(res, None)
 
-        import sys
-        if sys.version[:3] != '2.6':
-            output = memoryview(bytearray(16))
-            cipher = ChaCha20_Poly1305.new(key=key, nonce=nonce)
-            cipher.encrypt(pt, output=output)
-            self.assertEqual(ct, output)
-        
-            cipher = ChaCha20_Poly1305.new(key=key, nonce=nonce)
-            cipher.decrypt(ct, output=output)
-            self.assertEqual(pt, output)
+        output = memoryview(bytearray(16))
+        cipher = ChaCha20_Poly1305.new(key=key, nonce=nonce)
+        cipher.encrypt(pt, output=output)
+        self.assertEqual(ct, output)
+
+        cipher = ChaCha20_Poly1305.new(key=key, nonce=nonce)
+        cipher.decrypt(ct, output=output)
+        self.assertEqual(pt, output)
 
         cipher = ChaCha20_Poly1305.new(key=key, nonce=nonce)
         self.assertRaises(TypeError, cipher.encrypt, pt, output=b'0'*16)
-        
+
         cipher = ChaCha20_Poly1305.new(key=key, nonce=nonce)
         self.assertRaises(TypeError, cipher.decrypt, ct, output=b'0'*16)
 
         shorter_output = bytearray(7)
-        
+
         cipher = ChaCha20_Poly1305.new(key=key, nonce=nonce)
         self.assertRaises(ValueError, cipher.encrypt, pt, output=shorter_output)
-        
+
         cipher = ChaCha20_Poly1305.new(key=key, nonce=nonce)
         self.assertRaises(ValueError, cipher.decrypt, ct, output=shorter_output)
 
